﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ImageExtractor
{
    public partial class FormMain : Form
    {
        #region 운영서버
        private const string DB_ADDRESS = "127.0.0.1";
        private const string DB_NAME = "RTEGMS_HANPOONG";
        private const string DB_ID = "sa";
        private const string DB_PW = "!1HPpharm";
        #endregion

        #region 개발서버
        //private const string DB_ADDRESS = "sf.tascorp.co.kr,1455";
        //private const string DB_NAME = "RTEGMS_HANPOONG_MES";
        //private const string DB_ID = "sa";
        //private const string DB_PW = "!1Tascorp";        
        #endregion
        private const string TARGET_ROW_COUNT = "10";

        private SqlConnection _SqlConnection = new SqlConnection();
        private SqlCommand _SqlCommand = null;
        private string _ConnectionString = string.Empty;
        private string _AudittrailId = "0";
        private string _ImageDirectory = "D:\\images";
        private DateTime _StartTime = DateTime.Now;
        private int _DurationSeconds = 0;

        private BackgroundWorker _BackgroundWorker = new BackgroundWorker();

        public FormMain()
        {
            InitializeComponent();
            _ConnectionString = $"Server={DB_ADDRESS};database={DB_NAME};uid={DB_ID};pwd={DB_PW}";
            tb_TargetDirectory.Text = _ImageDirectory;

            _BackgroundWorker.WorkerReportsProgress = true;
            _BackgroundWorker.WorkerSupportsCancellation = true;
            _BackgroundWorker.DoWork += new DoWorkEventHandler(BackgroundWorker_DoWork);
            _BackgroundWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(BackgroundWorker_RunWorkerCompleted);
        }

        private void btn_extract_Click(object sender, EventArgs e)
        {
            lb_ExtractCount.Text = "0";
            ChangeEnabledButtons(false, true);
            _StartTime = DateTime.Now;
            ChangeTimerEnabled(true);

            _BackgroundWorker.RunWorkerAsync();
        }


        private void btn_stop_Click(object sender, EventArgs e)
        {
            ChangeEnabledButtons(true, false);
            ChangeTimerEnabled(false);

            _BackgroundWorker.CancelAsync();
        }

        private void ChangeEnabledButtons(bool isExtractEnabled, bool isStopEnabeld)
        {
            if (btn_extract.InvokeRequired)
            {
                Action changeEnabledButtons = delegate { ChangeEnabledButtons(isExtractEnabled, isStopEnabeld); };
                btn_extract.Invoke(changeEnabledButtons);
            }
            else
            {
                btn_extract.Enabled = isExtractEnabled;
                btn_stop.Enabled = isStopEnabeld;
            }
        }

        private void DisplayImageFileName(string fileName)
        {
            if (tb_CurrentImageName.InvokeRequired)
            {
                Action displayImageFileName = delegate { DisplayImageFileName(fileName); };
                tb_CurrentImageName.Invoke(displayImageFileName);
            }
            else
            {
                if (!string.IsNullOrEmpty(fileName))
                {
                    tb_CurrentImageName.Text = fileName;
                    lb_ExtractCount.Text = (Convert.ToInt32(lb_ExtractCount.Text) + 1).ToString();
                }
                else
                    tb_CurrentImageName.Text = "이미지 추출이 완료 되었습니다.";
            }
        }

        private void BackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            while (_AudittrailId != null && !_BackgroundWorker.CancellationPending)
            {
                Tuple<bool, List<string>> result = ExtractImage(SearchImage());
                if (result.Item1 && cb_DeleteAfterExtract.Checked)
                    DeleteImage(result.Item2);

                _AudittrailId = null;
                if (result.Item2 != null)
                    _AudittrailId = result.Item2[result.Item2.Count - 1];
            }
        }

        private void BackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ChangeEnabledButtons(true, false);
            ChangeTimerEnabled(false);
            GC.Collect();
        }

        private void ChangeTimerEnabled(bool enabled)
        {
            if (lb_DurationTime.InvokeRequired)
            {
                Action changeTimerEnabled = delegate { ChangeTimerEnabled(enabled); };
                lb_DurationTime.Invoke(changeTimerEnabled);
            }
            else
            {
                tmr_Duration.Enabled = enabled;
            }
        }

        private DataTable SearchImage()
        {
            if (_SqlCommand != null)
            {
                string query = @"
SELECT TOP @TARGET_ROW_COUNT item_cd, upper_image, audittrail_id
  FROM item_image
 WHERE item_cd like 'G2%'
   AND audittrail_id > @AUDITTRAIL_ID
ORDER BY audittrail_id
";
                query = query.Replace("@TARGET_ROW_COUNT", TARGET_ROW_COUNT);
                _SqlCommand.CommandText = query.Replace("@AUDITTRAIL_ID", _AudittrailId);
                SqlDataAdapter sd = new SqlDataAdapter(_SqlCommand);
                DataSet ds = new DataSet();
                sd.Fill(ds, "Result");

                return ds.Tables[0];
            }
            else
                return null;
        }

        private Tuple<bool, List<string>> ExtractImage(DataTable dataTable)
        {
            if (dataTable == null || dataTable.Rows.Count == 0)
            {
                return new Tuple<bool, List<string>>(false, null);
            }
            else
            {
                List<string> audittrailList = new List<string>();
                try
                {
                    foreach (DataRow dataRow in dataTable.Rows)
                    {
                        byte[] fileBytes = dataRow["upper_image"] as byte[];
                        string fileName = $"{dataRow["item_cd"]}.png";
                        string audittrailId = dataRow["audittrail_id"].ToString();

                        string filePath = Path.Combine(_ImageDirectory, fileName);
                        if (fileBytes != null)
                            File.WriteAllBytes(filePath, fileBytes);

                        DisplayImageFileName(fileName);

                        if (audittrailId != null)
                            audittrailList.Add(audittrailId);
                    }


                    return new Tuple<bool, List<string>>(true, audittrailList);
                }
                catch
                {
                    return new Tuple<bool, List<string>>(false, audittrailList);
                }
            }
        }

        private void DeleteImage(List<string> audittrailList)
        {
            if (_SqlCommand != null && audittrailList != null)
            {
                string audittrailIds = string.Empty;
                foreach (string audittrailId in audittrailList)
                {
                    audittrailIds += string.Format("{0}{1}", audittrailIds == string.Empty ? string.Empty : ",", audittrailId);
                }

                string query = @"
DELETE item_image
 WHERE audittrail_id in (@AUDITTRAIL_ID)
";
                query = query.Replace("@TARGET_ROW_COUNT", TARGET_ROW_COUNT);
                query = query.Replace("@AUDITTRAIL_ID", audittrailIds);
                _SqlCommand.CommandText = query;
                SqlDataAdapter sd = new SqlDataAdapter(_SqlCommand);
                DataSet ds = new DataSet();
                sd.Fill(ds, "Result");
            }
        }

        private void btn_connect_Click(object sender, EventArgs e)
        {
            if (_SqlCommand == null)
            {
                if (DbConnect())
                {
                    ChangeEnabledButtons(true, false);
                }
            }
            else if (DbDisconnect())
            {
                ChangeEnabledButtons(false, false);
            }
        }

        private bool DbConnect()
        {
            try
            {
                _SqlConnection.ConnectionString = _ConnectionString;
                _SqlConnection.Open();

                _SqlCommand = new SqlCommand();
                _SqlCommand.Connection = _SqlConnection;
                _SqlCommand.CommandTimeout = 0;

                btn_connect.Text = "Disconnect";

                return true;
            }
            catch (Exception ex)
            {
                _SqlConnection.Close();
                MessageBox.Show(ex.Message);

                return false;
            }
        }

        private bool DbDisconnect()
        {
            try
            {
                _SqlConnection.Close();
                _SqlCommand = null;

                btn_connect.Text = "Connect";

                return true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return false;
            }
        }

        private void btn_TargetDirectory_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            if (DialogResult.OK == dialog.ShowDialog())
            {
                _ImageDirectory = dialog.SelectedPath;
                tb_TargetDirectory.Text = _ImageDirectory;
            }
        }

        private void tmr_Duration_Tick(object sender, EventArgs e)
        {
            DateTime currentTime = DateTime.Now;
            int totalSeconds = (int)(currentTime - _StartTime).TotalSeconds;
            if (totalSeconds != _DurationSeconds)
            {
                _DurationSeconds = totalSeconds;

                double _totalSeconds = _DurationSeconds;
                int hours = (int)(_totalSeconds / 3600);
                _totalSeconds = _totalSeconds % 3600;
                int minutes = (int)(_totalSeconds / 60);
                _totalSeconds = _totalSeconds % 60;
                int seconds = (int)(_totalSeconds);

                lb_DurationTime.Text = string.Format("{0:D2}:{1:D2}:{2:D2}", hours, minutes, seconds);
                CalculateCountPerSecond();
            }
        }

        private void CalculateCountPerSecond()
        {
            double count = Convert.ToInt32(lb_ExtractCount.Text);
            lb_CountPerSecond.Text = (Math.Truncate((count / (double)_DurationSeconds) * 100) / 100).ToString();
        }
    }
}
